#!/usr/bin/env python

import sys
import time
from phpcode import codegen

class builtin:
	def __init__(self, name, handler, description = ""):
		self.name = name
		self.handler = handler
		self.description = (15-len(self.name))*" "+"-   "+description

class usershell:
	prompt = "pshcli> "

	def __init__(self, dumper = None):
		self.quiet = False
		self.dumper = None
		self._builtin_dump(["dump ",dumper])

		self.builtins = {
				"help" : builtin("help", self._builtin_list_builtins, "show help"),
				"quit" : builtin("quit", self._builtin_exit, "quit psh"),
				"exit" : builtin("exit", self._builtin_exit, "same as quit"),
				"dump" : builtin("dump", self._builtin_dump, "set dumpfile, arg: file"),
				"quiet": builtin("quiet",self._builtin_quiet, "toggle console output"),
				"comment": builtin("comment",self._builtin_comment, "write a formated comment into your dumpfile"),
						}

	def read(self):
		sys.stdout.write(self.prompt)
		return raw_input()

	def write(self, data, nodump = 0, stderr=0):
		if not self.quiet:
			if stderr:			
				sys.stderr.write(data)
			else:
				sys.stdout.write(data)
		if not nodump:
			if self.dumper is not None:
				self.dumper.write(data)

	def writeline(self, data, nodump = 0, stderr=0):
		self.write(data+"\n", nodump, stderr)

	def writetitle(self, data):
		self.write("\n"+data+"\n")

	def emptylines(self, num):
		self.write("\n"*num)

	def fatal(self, string):
		self.writeline("FATAL: "+string, nodump=1, stderr=1)
		exit(1)

	def warn(self, string):
		self.writeline("WARNING: "+string, nodump=1, stderr=1)

	def loop(self, evaluator):
		self.evaluator = evaluator
		while 1:
			try:
				input = self.read()
				if self.builtin(input):
					continue
				result = self.evaluator.eval(input)
				self.write(result.read())
			except EOFError:
				self.writeline("")
				break;

	def builtin(self, string):
		try:
			cmd = string.split(" ")
			self.builtins[cmd[0]].handler(cmd)
			return True
		except KeyError:
			return False

	def _builtin_list_builtins(self, cmd):
		self.writetitle("List of Builtin Commands:")
		for builtin in self.builtins.itervalues():
			self.writeline(builtin.name+" "+builtin.description)
		self.writeline("")


	def _builtin_exit(self, cmd):
		raise EOFError("exit")

	def _builtin_dump(self, cmd):
		if len(cmd) > 1:
			dumpfile = cmd[1]
			try:
				self.dumper = open(dumpfile,"a")
			except:
				self.warn("could not set dumper, session will not be dumped!")
		else:
			if self.dumper is not None:
				self.writeline("current dumper: "+self.dumper.name)
			else:
				self.writeline("No dumper installed!")

	def _builtin_quiet(self, cmd):
		self.quiet = not self.quiet

	def _builtin_comment(self, cmd):
		self.write("### COMMENT "+time.asctime()+" ### ")
		for word in cmd[1:]:
			self.write(word+" ")
		self.emptylines(1)

	def __del__(self):
		if self.dumper is not None:
			self.dumper.close()

